/***************************************************
* By CCSC compiler
*  File Name: 20220117_K653C_MK158D_red_1579.c
 * rev 1.0 20220117
*  Description: PIR sensor MP3 player
 * RB7 O TX FOR MK156B CONTROL
 * RB6 I NA
 * RB5 I RX
 * RB4 I NA
 * RA5 I NA
 * RA4 I NIGHT=0 pullups
 * RA3 I SW4-8 pullups
 * RA2 I PIR, int
 * RA1 I SW4-2 pullups
 * RA0 I SW4-1 pullups
 * RC5 O LED
 * RC4 I BUSY FOR MK156B active low
 * RC3 I ADC LDR
 * RC2 O POWER
 * RC1 O POWER FOR LDR
 * RC0 I SW4-4 pullups
*  COPYRIGHT 2023 MYCOMKITS.COM, owned by CNET LIMITED
* The copyright of this program belongs to the producer "MYCOMKITS.COM management company CNET LIMITED". 
* We do not abandon copyright, but you can use and change the program in learning using this program. 
***************************************************/
//
#include <16F1579.h>
#DEVICE ADC=8
#fuses	INTRC_IO, NOWDT, NOMCLR, NOBROWNOUT, PUT, NOLVP, NOPROTECT,
#fuses	NOCLKOUT, NOWRT
#use	delay(CLOCK = 4000000)
#use	rs232(BAUD=9600, XMIT=PIN_B7, RCV=PIN_B5)

#INCLUDE<stdlib.h>
//
#use	fast_io(a)
#use	fast_io(b)
#use	fast_io(c)
//
#define Amode 0xFF			//port A all input except A5
#define Bmode 0x7F			//port B
#define Cmode 0xD9			//port C all input, except C1,2
#define MODE PIN_A4
#define BUSY PIN_C4 // active low
#define PIR PIN_A2 // active high
#define LDR_POWER PIN_C1 // active high
#define MK156_POWER PIN_C2 // active high
#define vol_p PIN_C7 // active low
#define vol_m PIN_C6 // active low
#define LED PIN_C5 // active high

// variables
int value;
int file_number;
int enable_flag = 0;
int wakeup=0;
int volume_value;
int volume_value_init=0x0F;
int file_value;
int file_value_init=0x01;
long value_memory_address=0x1F80;
int32 seconds_origin=0;
int32 seconds=0;
int max_file=15;
char string[20];

// prototyping
#separate
void get_sensor();
#separate
void initializing();
#separate
void mode_check();
#separate
void play_one();
#separate
void play_stop();
#separate
void get_number();
#separate
void get_switch();
#separate
void set_volume();
#separate
void get_maxfile();
#separate
void play_prev();
#separate
void play_next();
#separate
void led2time();

// main function
void main()
{
    initializing();

    // main loop
    while(1)
    {
        if(wakeup==1)
        {
            disable_interrupts(GLOBAL);
            port_a_pullups(0x1B);   //except RA2,5
            port_c_pullups(0xC1);   //
            delay_ms(30);
            mode_check();   //day or night
            output_high(LED);
            output_high(PIN_B7); //RS232 on

            if(enable_flag == 1)
            {
                output_high(MK156_POWER);    //power on
                delay_ms(1500);
                if(input(BUSY) == 1)    //active low
                {
                    get_number();
                    volume_value = read_program_eeprom(value_memory_address);
                    set_volume();
                    //specify file
                    putc(0x7E);
                    putc(0xFF);
                    putc(0x06);
                    putc(0x0F);
                    putc(0x00);
                    putc(0x01); //folder 01
                    putc(file_number); //file number
                    putc(0xEF);
                    delay_ms(100);
                    play_one();

                   while(input(BUSY) == 0)  //active low 
                   {
                       get_switch();
                   }
                   //
                   output_low(MK156_POWER);    //power off
                   
                }
            }
        }
        
        port_a_pullups(0x00);   //except RA2,5
        port_b_pullups(0x00); 
        port_c_pullups(0x00); 
        output_low(LDR_POWER);
        output_low(MK156_POWER);    //power off
        output_low(PIN_B7); //RS232 off
        
        while(input(PIN_A2)==1) {}
        output_low(LED);
        wakeup=0;
        clear_interrupt(INT_EXT);
        enable_interrupts(GLOBAL);
        sleep();
    }
}
//
#separate
void get_sensor()
{
    set_adc_channel(7);	//RC3, red, must wait 65u
    delay_us(100);
    value = read_adc();	//0 to 255
    if(value < 5) value = 0;	// 2 to 255
}
//
#separate
void initializing()
{
	set_tris_a(Amode);	//
    set_tris_b(Bmode);	//
	set_tris_c(Cmode);	//
	//
	setup_comparator(NC_NC_NC_NC);
	// A/D converter initialize
	setup_adc_ports(sAN7 | VSS_VDD);	// RC3, gnd to 5v
	setup_adc(ADC_CLOCK_DIV_8);			// 2usec
    
    volume_value = read_program_eeprom(value_memory_address);
    if((volume_value > 0x1E) || (volume_value == 0))
    {
        volume_value = volume_value_init;
        write_program_eeprom(value_memory_address, volume_value);
    }
    
    file_value = read_program_eeprom(value_memory_address + 1);
    if(file_value > 0x0f)
    {
        file_value = file_value_init;
        write_program_eeprom(value_memory_address + 1, file_value);
    }
    delay_ms(1000);
    set_volume();
    
    output_low(LDR_POWER);  //active high
    output_low(MK156_POWER);   //active high
    output_low(LED);
    
    port_a_pullups(0);
    port_b_pullups(0);
    port_c_pullups(0);
    
    enable_interrupts(INT_EXT);
    ext_int_edge(L_TO_H);
    enable_interrupts(GLOBAL);

}

#INT_EXT
void ext_isr(void)
{
    delay_ms(30);
    wakeup=1;
}
//
#separate
void mode_check()
{
    if(input(MODE) == 0)    // night
    {
        output_high(LDR_POWER);
        delay_ms(100);
        get_sensor();
        if(value < 0x7F) enable_flag = 1;
        else enable_flag = 0;
    }
    else
    {
        enable_flag = 1;
        output_low(LDR_POWER);  //not available
    }
    
}
//
#separate
void play_one()
 {
    putc(0x7E);
    putc(0xFF);
    putc(0x06);
    putc(0x0D);
    putc(0x00);
    putc(0x00);
    putc(0x00);
    putc(0xEF);
    //delay_ms(1000);
}
//
#separate
void play_stop()
{
    putc(0x7E);
    putc(0x02);
    putc(0x0E); // stop
    putc(0xEF);
    delay_ms(10);
}
//
#separate
void get_number()
{
    int file_tempo;
    //file_tempo = file_number;
    file_number = input_a() & 0x0B; // 0 to 11, except RA2
    if(input(PIN_C0)==1) file_number = file_number + 4;
    file_number = 15 - file_number; //for white
    if(file_number == 0) 
    {
        get_maxfile();
        file_tempo = read_program_eeprom(value_memory_address + 1);
        file_tempo = file_tempo + 1;
        if(file_tempo > max_file)
        {
            file_tempo = 1;
        }
        file_number = file_tempo;
        write_program_eeprom(value_memory_address + 1, file_number);
    }
}

#separate
void get_switch()
{
    if(input(vol_m)==0)
    {
        delay_ms(20);
        if(input(vol_m)==0)
        {
            led2time();
            while(input(vol_m)==0)
            {
                if(volume_value > 0)
                {
                    volume_value=volume_value-1;
                    set_volume();
                    write_program_eeprom(value_memory_address, volume_value);
                    delay_ms(50);
                }
            }
        }

    }
    
    if(input(vol_p)==0)
    {
        delay_ms(20);
        if(input(vol_p)==0)
        {
            while(input(vol_p)==0)
            {
                if(volume_value < 0x1E)
                {
                    volume_value=volume_value+1;
                    set_volume();
                    write_program_eeprom(value_memory_address, volume_value);
                    delay_ms(50);
                }
            }
        }

    }
}

#separate
void set_volume()
{
        //specify volume
    putc(0x7E);
    putc(0xFF);
    putc(0x06);
    putc(0x06);
    putc(0x00);
    putc(0x00);
    putc(volume_value);
    putc(0xEF);
    delay_ms(20);
 
}

#separate
void get_maxfile()
{
    //7E FF 06 4E 00 00 01 EF
    putc(0x7E);
    putc(0xFF);
    putc(0x06);
    putc(0x4E);
    putc(0x00);
    putc(0x00);
    putc(0x01);
    putc(0xEF);
    int i;
    int tempo_data;
    max_file = 3;
    for(i=0; i<7; i++)
    {
        tempo_data = getc();
        max_file = tempo_data;   //number of files
    }
}

#separate
void play_prev()
{
    putc(0x7E);
    putc(0xFF);
    putc(0x06);
    putc(0x02);
    putc(0x00);
    putc(0x00);
    putc(0x00);
    putc(0xEF);
    delay_ms(20);
    while(input(vol_m)==0) {}
}

#separate
void play_next()
{
    putc(0x7E);
    putc(0xFF);
    putc(0x06);
    putc(0x01);
    putc(0x00);
    putc(0x00);
    putc(0x00);
    putc(0xEF);
    delay_ms(20);
    while(input(vol_p)==0) {}
}

#separate
void led2time()
{
    int i;
    for(i=0;i<2;i++)
    {
        output_high(LED);
        delay_ms(200);
        output_low(LED);
        delay_ms(200);
    }
    output_low(LED);
}
